// Copyright (c) 2000-2008 Synology Inc. All rights reserved.
#ifndef __SYNOLIB_H_
#define __SYNOLIB_H_

#include <linux/workqueue.h>
#include <linux/timer.h>
#include <linux/spinlock.h>
#include <linux/jiffies.h>
#include <linux/module.h>


#ifdef  SYNO_DEBUG_FLAG
extern int syno_temperature_debug;
#endif

typedef struct _tag_SynoWorkQueue{
	struct workqueue_struct	*pWorkQueue;
	struct work_struct _worker;
}SynoWorkQueue;

typedef struct _tag_SynoAsyncOperation{
	struct timer_list	asyncStartTimer;
	SynoWorkQueue	*pSynoWorkQueue;

	/*
	* period function, and his ownly data pointer.
	* period_in_sec is the periodly execution unit.
	* stopAsyncOperation will cause stop period execution
    */
	void (*period_func)(void *data);
	void *period_func_data;	
	unsigned long	period_in_sec;
	u8	stopAsyncOperation;
		
	spinlock_t	syno_async_lock;	
}SYNOASYNCOPERATION;


int SynoAsyncOperationInit(SYNOASYNCOPERATION *pSynoAsyncOp, struct workqueue_struct *pWorkQueue,
							 void (*period_func)(void *data), void *user_data, unsigned long period_in_sec, u8 needScheduler);
extern void SynoAsyncOperationCleanUp(SYNOASYNCOPERATION *pSynoAsyncOp);
extern void SynoAsyncOperationPause(SYNOASYNCOPERATION *pSynoAsyncOp);
extern void SynoAsyncOperationResume(SYNOASYNCOPERATION *pSynoAsyncOp);
extern void SynoAsyncOperationModifyPeriod(SYNOASYNCOPERATION *pSynoAsyncOp, unsigned long period_in_sec);

extern asmlinkage int SynoPrintk(u8 direct_print, const char *fmt, ...);

#define SYNO_AYNC_OP_INIT(pSynoAsyncOp, period_func, user_data, period_in_sec)	SynoAsyncOperationInit(pSynoAsyncOp, NULL, period_func, user_data, period_in_sec, 0)
#define SYNO_SCHED_ASYNC_OP_INIT(pSynoAsyncOp, period_func, user_data, period_in_sec)	SynoAsyncOperationInit(pSynoAsyncOp, NULL, period_func, user_data, period_in_sec, 1)
#define SYNO_SCHED_ASYNC_OP_INIT_WITH_WORKQUEUE(pSynoAsyncOp, pWorkQueue, period_func, user_data, period_in_sec)	SynoAsyncOperationInit(pSynoAsyncOp, pWorkQueue, period_func, user_data, period_in_sec, 1)

/*
* Because these symbols are GPL ONLY, so we can't use it in synobios directly
* so we re-export these symbol here.
*/
EXPORT_SYMBOL(__create_workqueue);
EXPORT_SYMBOL(queue_work);
EXPORT_SYMBOL(queue_delayed_work);
EXPORT_SYMBOL(flush_workqueue);
EXPORT_SYMBOL(destroy_workqueue);

#ifdef SYNO_DEBUG_FLAG
void syno_do_hibernation_log(const char __user *filename);
#endif

#endif //__SYNOLIB_H_
